@@ -290,7 +290,7 @@ impl State {
             w.G(&mut t, 3);
             w.expand();
             w.G(&mut t, 4);
-            t.add(&self);
+            t.add(self);
             self.0 = t.0;
             input = &input[128..];
             inlen -= 128;
@@ -375,11 +375,11 @@ impl Default for Hash {
     }
 }
 
-#[cfg(feature = "traits")]
+#[cfg(feature = "traits09")]
 mod digest_trait {
     use super::Hash;
-    use digest::consts::{U128, U64};
-    use digest::{BlockInput, FixedOutputDirty, Reset, Update};
+    use digest09::consts::{U128, U64};
+    use digest09::{BlockInput, FixedOutputDirty, Output, Reset, Update};
 
     impl BlockInput for Hash {
         type BlockSize = U128;
@@ -394,7 +394,69 @@ mod digest_trait {
     impl FixedOutputDirty for Hash {
         type OutputSize = U64;
 
-        fn finalize_into_dirty(&mut self, out: &mut digest::Output<Self>) {
+        fn finalize_into_dirty(&mut self, out: &mut Output<Self>) {
+            let h = self.finalize();
+            out.copy_from_slice(&h);
+        }
+    }
+
+    impl Reset for Hash {
+        fn reset(&mut self) {
+            *self = Self::new();
+        }
+    }
+}
+
+/// Wrapped `Hash` type for the `Digest` trait.
+#[cfg(feature = "traits010")]
+pub type WrappedHash = digest010::core_api::CoreWrapper<Hash>;
+
+#[cfg(feature = "traits010")]
+mod digest_trait010 {
+    use super::Hash;
+    use core::fmt;
+    use digest010::{
+        block_buffer::Eager,
+        consts::{U128, U64},
+        core_api::{
+            AlgorithmName, Block, BlockSizeUser, Buffer, BufferKindUser, FixedOutputCore,
+            OutputSizeUser, Reset, UpdateCore,
+        },
+        HashMarker, Output,
+    };
+
+    impl AlgorithmName for Hash {
+        fn write_alg_name(f: &mut fmt::Formatter<'_>) -> fmt::Result {
+            f.write_str("Sha512")
+        }
+    }
+
+    impl HashMarker for Hash {}
+
+    impl BufferKindUser for Hash {
+        type BufferKind = Eager;
+    }
+
+    impl BlockSizeUser for Hash {
+        type BlockSize = U128;
+    }
+
+    impl OutputSizeUser for Hash {
+        type OutputSize = U64;
+    }
+
+    impl UpdateCore for Hash {
+        #[inline]
+        fn update_blocks(&mut self, blocks: &[Block<Self>]) {
+            for block in blocks {
+                self._update(block);
+            }
+        }
+    }
+
+    impl FixedOutputCore for Hash {
+        fn finalize_fixed_core(&mut self, buffer: &mut Buffer<Self>, out: &mut Output<Self>) {
+            self._update(buffer.get_data());
             let h = self.finalize();
             out.copy_from_slice(&h);
         }
@@ -534,11 +596,11 @@ pub mod sha384 {
         }
     }
 
-    #[cfg(feature = "traits")]
-    mod digest_trait {
+    #[cfg(feature = "traits09")]
+    mod digest_trait09 {
         use super::Hash;
-        use digest::consts::{U128, U48};
-        use digest::{BlockInput, FixedOutputDirty, Reset, Update};
+        use digest09::consts::{U128, U48};
+        use digest09::{BlockInput, FixedOutputDirty, Output, Reset, Update};
 
         impl BlockInput for Hash {
             type BlockSize = U128;
@@ -553,7 +615,69 @@ pub mod sha384 {
         impl FixedOutputDirty for Hash {
             type OutputSize = U48;
 
-            fn finalize_into_dirty(&mut self, out: &mut digest::Output<Self>) {
+            fn finalize_into_dirty(&mut self, out: &mut Output<Self>) {
+                let h = self.finalize();
+                out.copy_from_slice(&h);
+            }
+        }
+
+        impl Reset for Hash {
+            fn reset(&mut self) {
+                *self = Self::new();
+            }
+        }
+    }
+
+    /// Wrapped `Hash` type for the `Digest` trait.
+    #[cfg(feature = "traits010")]
+    pub type WrappedHash = digest010::core_api::CoreWrapper<Hash>;
+
+    #[cfg(feature = "traits010")]
+    mod digest_trait010 {
+        use super::Hash;
+        use core::fmt;
+        use digest010::{
+            block_buffer::Eager,
+            consts::{U128, U48},
+            core_api::{
+                AlgorithmName, Block, BlockSizeUser, Buffer, BufferKindUser, FixedOutputCore,
+                OutputSizeUser, Reset, UpdateCore,
+            },
+            HashMarker, Output,
+        };
+
+        impl AlgorithmName for Hash {
+            fn write_alg_name(f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                f.write_str("Sha384")
+            }
+        }
+
+        impl HashMarker for Hash {}
+
+        impl BufferKindUser for Hash {
+            type BufferKind = Eager;
+        }
+
+        impl BlockSizeUser for Hash {
+            type BlockSize = U128;
+        }
+
+        impl OutputSizeUser for Hash {
+            type OutputSize = U48;
+        }
+
+        impl UpdateCore for Hash {
+            #[inline]
+            fn update_blocks(&mut self, blocks: &[Block<Self>]) {
+                for block in blocks {
+                    self._update(block);
+                }
+            }
+        }
+
+        impl FixedOutputCore for Hash {
+            fn finalize_fixed_core(&mut self, buffer: &mut Buffer<Self>, out: &mut Output<Self>) {
+                self._update(buffer.get_data());
                 let h = self.finalize();
                 out.copy_from_slice(&h);
             }
